from typing import NotRequired, List, Any, Dict, TypedDict

from debputy.manifest_parser.base_types import DebputyParsedContent, OctalMode
from debputy.manifest_parser.declarative_parser import parser_from_typed_dict
from debputy.plugin.api.impl_types import KnownPackagingFileInfo


class PPFReferenceDocumentation(TypedDict):
    description: NotRequired[str]
    format_documentation_uris: NotRequired[List[str]]


class PackagerProvidedFileJsonDescription(DebputyParsedContent):
    stem: str
    installed_path: str
    default_mode: NotRequired[OctalMode]
    default_priority: NotRequired[int]
    allow_name_segment: NotRequired[bool]
    allow_architecture_segment: NotRequired[bool]
    reference_documentation: NotRequired[PPFReferenceDocumentation]


class ManifestVariableJsonDescription(DebputyParsedContent):
    name: str
    value: str
    reference_documentation: NotRequired[str]


class PluginJsonMetadata(DebputyParsedContent):
    api_compat_version: int
    module: NotRequired[str]
    plugin_initializer: NotRequired[str]
    packager_provided_files: NotRequired[List[Any]]
    manifest_variables: NotRequired[List[Any]]
    known_packaging_files: NotRequired[List[Any]]


PLUGIN_METADATA_PARSER = parser_from_typed_dict(PluginJsonMetadata)
PLUGIN_PPF_PARSER = parser_from_typed_dict(PackagerProvidedFileJsonDescription)
PLUGIN_MANIFEST_VARS_PARSER = parser_from_typed_dict(ManifestVariableJsonDescription)
PLUGIN_KNOWN_PACKAGING_FILES_PARSER = parser_from_typed_dict(KnownPackagingFileInfo)
